"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Align = ["start", "center", "end"];
exports.Anchor = [
    "top_left", "top_center", "top_right",
    "center_left", "center", "center_right",
    "bottom_left", "bottom_center", "bottom_right",
];
exports.AngleUnits = ["deg", "rad"];
exports.BoxOrigin = ["corner", "center"];
exports.ButtonType = ["default", "primary", "success", "warning", "danger"];
exports.Dimension = ["width", "height"];
exports.Dimensions = ["width", "height", "both"];
exports.Direction = ["clock", "anticlock"];
exports.Distribution = ["uniform", "normal"];
exports.FontStyle = ["normal", "italic", "bold", "bold italic"];
exports.HTTPMethod = ["POST", "GET"];
exports.HexTileOrientation = ["pointytop", "flattop"];
exports.HoverMode = ["mouse", "hline", "vline"];
exports.LatLon = ["lat", "lon"];
exports.LegendClickPolicy = ["none", "hide", "mute"];
exports.LegendLocation = exports.Anchor;
exports.LineCap = ["butt", "round", "square"];
exports.LineJoin = ["miter", "round", "bevel"];
exports.LinePolicy = ["prev", "next", "nearest", "interp", "none"];
exports.Location = ["above", "below", "left", "right"];
exports.Logo = ["normal", "grey"];
exports.MarkerType = [
    "asterisk", "circle", "circle_cross", "circle_x", "cross",
    "dash", "diamond", "diamond_cross", "hex", "inverted_triangle",
    "square", "square_cross", "square_x", "triangle", "x",
];
exports.Orientation = ["vertical", "horizontal"];
exports.OutputBackend = ["canvas", "svg", "webgl"];
exports.PaddingUnits = ["percent", "absolute"];
exports.Place = ["above", "below", "left", "right", "center"];
exports.PointPolicy = ["snap_to_data", "follow_mouse", "none"];
exports.RadiusDimension = ["x", "y", "max", "min"];
exports.RenderLevel = ["image", "underlay", "glyph", "annotation", "overlay"];
exports.RenderMode = ["canvas", "css"];
exports.RoundingFunction = ["round", "nearest", "floor", "rounddown", "ceil", "roundup"];
exports.Side = ["above", "below", "left", "right"];
exports.SizingMode = ["stretch_width", "stretch_height", "stretch_both", "scale_width", "scale_height", "scale_both", "fixed"];
exports.SliderCallbackPolicy = ["continuous", "throttle", "mouseup"];
exports.Sort = ["ascending", "descending"];
exports.SpatialUnits = ["screen", "data"];
exports.StartEnd = ["start", "end"];
exports.StepMode = ["after", "before", "center"];
exports.TapBehavior = ["select", "inspect"];
exports.TextAlign = ["left", "right", "center"];
exports.TextBaseline = ["top", "middle", "bottom", "alphabetic", "hanging", "ideographic"];
exports.TickLabelOrientation = ["vertical", "horizontal", "parallel", "normal"];
exports.TooltipAttachment = ["horizontal", "vertical", "left", "right", "above", "below"];
exports.UpdateMode = ["replace", "append"];
exports.VerticalAlign = ["top", "middle", "bottom"];
